/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "mcp16331.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define MCP16331_DUMMY 0

// ----------------------------------------------- PRIVATE FUNCTION DECLARATION 

static void dac_output ( mcp16331_t *ctx, uint16_t value_dac );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void mcp16331_cfg_setup ( mcp16331_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->adc_out = HAL_PIN_NC;
    cfg->en = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

MCP16331_RETVAL mcp16331_init ( mcp16331_t *ctx, mcp16331_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return MCP16331_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, MCP16331_DUMMY );

    // Output pins 
    
    digital_out_init( &ctx->en, cfg->en );


    // Input pins

    digital_in_init( &ctx->adc_out, cfg->adc_out );

    return MCP16331_OK;
}

void mcp16331_generic_transfer ( mcp16331_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );    
}

void mcp16331_set_vout ( mcp16331_t *ctx, uint16_t millivolts_vout )
{
     uint16_t temp = millivolts_vout;
     
     temp -= 2000;                  // 2000 mV is 0, 12000 mV is 10000
     temp /= 245;
     temp *= 100;

     temp = 4095 - temp;
     dac_output( ctx, temp );
}

// ------------------------------------------------------------ PRIVATE FUNCTION 

static void dac_output ( mcp16331_t *ctx, uint16_t value_dac )
{
    uint8_t temp;

    temp = ( ( value_dac >> 8) & 0x0F ) | 0x30;

    spi_master_start( &ctx->spi );                
    spi_master_write( &ctx->spi, temp );
    spi_master_write( &ctx->spi, value_dac );
    spi_master_stop( &ctx->spi );                 
}

// ------------------------------------------------------------------------- END

